import json

from jelm import read_json_dump, Jelm


def test_json_dump_read():

    test_dic = {
        'metadata': {'author': 'me'},
        'objects': []
    }

    dump = json.dumps(test_dic)

    jelm = read_json_dump(dump)

    assert isinstance(jelm, Jelm)



import json

from typing import Optional, Union


class Edge:

    def __init__(self,
                 source: str,
                 target: str,
                 id: Optional[str] = None,
                 attributes: Optional[dict] = None):

        self.source = source
        self.target = target
        self.id = id
        self.attributes = attributes or {}

    def get_dict(self) -> dict:
        optionals = {
            k: self.__getattribute__(k)
            for k in ['id', 'attributes'] if self.__getattribute__(k)
        }
        return {
            'type': 'edge',
            'source': self.source,
            'target': self.target,
            **optionals
        }


class Node:

    def __init__(self,
                 id: str,
                 attributes: Optional[dict] = None):
        self.id = id
        self.attributes = attributes or {}

    def get_dict(self) -> dict:
        if self.attributes:
            optionals = {'attributes': self.attributes}
        else:
            optionals = {}
        return {
            'type': 'edge',
            'id': self.id,
            **optionals
        }


class Jelm:

    def __init__(self,
                 metadata: Optional[dict]=None,
                 objects: Optional[list]=None,
                 **kwargs):

        self.metadata = metadata or {}
        self.objects = objects or []

        if kwargs:
            raise ValueError("Tried to create jelm object with additional kwargs {}"
                             .format(kwargs.keys()))

    def get_dict(self) -> dict:
        return {
            'metadata': self.metadata,
            'objects': self.objects
        }

    def get_json(self) -> str:
        return json.dumps(self.get_dict())

    def add_object(self, obj: Union[dict, Edge, Node]):

        if isinstance(obj, dict):

            try:
                obj_type = obj.pop('type')
            except KeyError:
                raise ValueError("if dict is given, 'type' key needs to be set! (to either node or edge)")

            if obj_type == 'edge':
                parsed_obj = Edge(**obj).get_dict()
            elif obj_type == 'node':
                parsed_obj = Node(**obj).get_dict()
            else:
                raise ValueError("object type needs to be either node or edge it is {}".format(obj_type))
        else:
            parsed_obj = obj.get_dict()

        self.objects.append(parsed_obj)



import pytest

from jelm import Jelm


def test_init():

    jelm = Jelm(metadata={'author': 'John Doe'},
                objects=[])

    with pytest.raises(ValueError):
        jelm2 = Jelm(bad_kwarg="fing")

    assert isinstance(jelm.objects, list)
    assert isinstance(jelm.metadata, dict)


def test_get():

    jelm = Jelm()

    assert isinstance(jelm.get_dict(), dict)
    assert isinstance(jelm.get_json(), str)


def test_add_object():

    jelm = Jelm()

    jelm.add_object({'type': 'edge',
                     'source': 'n1',
                     'target': 'n2'})

    jelm.add_object({'type': 'node',
                     'id': 'n1'})

    from jelm.core.jelm_class import Node

    jelm.add_object(Node(id='n2'))

    jelm.add_object(Node(id='n3',
                         attributes={'priority': 'low'}))

    with pytest.raises(ValueError):
        jelm.add_object({'no': 'type'})

    with pytest.raises(ValueError):
        jelm.add_object({'type': 'wrong'})



from setuptools import find_packages, setup
import jelm

pkg_name = 'jelm'

version = jelm.__version__


with open("README.md") as fp:
    long_description = fp.read()

setup(
    name=pkg_name,
    version=version,
    description="python graph description",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license='MIT',
    classifiers=[
        "License :: OSI Approved :: MIT License",
    ],
    url='https://github.com/endremborza/{}'.format(pkg_name),
    keywords='network graph json edgelist framework',
    author='Endre Márk Borza',
    author_email='endremborza@gmail.com',
    packages=find_packages(),
    include_package_data=True,
    python_requires='>=3.6',
)
